// Include Standard files
#include "define.h"
#include "delay.h"
#include "time_slot.h"
#include "spi.h"                    // 9-bit SPI routines using standard I/O pins
#include "font5x7.h"
#include "font8x14.h"
#include <math.h>

extern signed char CursorX;                       // Pixel cursor for drawing characters
extern signed char CursorY; 

unsigned char const Strings[5][18]=
{
" Henrique    Z",
"'            ~  Z",
"  Carlos Magno  Z",
"PSoCLCD_C128x128Z",
"   Full Color!  Z" 
}; 

// Cdigo de inicializao
unsigned int const DataInit1[12]={0xFDFD,0xFDFD,0xEF00,0xEE04,0x1B04,0xFEFE,0xFEFE,0xEF90,0x4A04,0x7F3F,0xEE04,0x4306};
unsigned int const DataInit2[23]={0xEF90,0x0983,0x0800,0x0BAF,0x0A00,0x0500,0x0600,0x0700,0xEF00,0xEE0C,0xEF90,0x0080,
0xEFB0,0x4902,0xEF00,0x7F01,0xE181,0xE202,0xE276,0xE183,0x8001,0xEF90,0x00};

//##################################
//#  LCD Lschen - Apagar display  #
//##################################

void S65_cls(double Bcol){
unsigned int  Platz, W1;

	RS_1;
	spi(0xEF90);
	spi(0x500);
	spi(0x600);
	spi(0x700);
	Platz = 132 * 176;
	RS_0;
	for(W1=0;W1<=Platz;W1++)
		spi(Bcol);
	RS_1;
	
}

void S65_init(void)
{
unsigned char i;
	RESET_1;
	delay_ms(10);
	RESET_0;                                                     //Reset
	CS_1;
	SCLK_1;
	SDATA_1;
	RS_1;
	
	delay_ms(1);
	RESET_1;                                                     //Reset Ende
	delay_ms(10);
	
	
	CS_1;                                                     //CS select, Start
	delay_ms(1);
	RS_1;

//#######################################################
//      Displayinitialisierung - Inicializao do display
//#######################################################

	for(i=0;i<12;i++)
		spi(DataInit1[i]);
	
	delay_ms(7);      //diese Zeit ist sehr wichtig, sollte keinesfalls unterschritten werden
					  //este tempo  muito importante - no deve ser menor que 7ms
	
	for(i=0;i<23;i++)
		spi(DataInit2[i]);

	RS_0;
}


//######################################################
//#                 Set Pixel -> Pset                #
//######################################################

void S65_pset(unsigned char Lx, unsigned char Ly, double Color){

	#ifdef LANDSCAPE
	RS_1;
	spi(0x600 + Ly);
	spi(0x700 + Lx);
	RS_0;
	spi(Color);
	#else
	#ifdef PORTRAIT
	RS_1;
	spi(0x600 + Lx);
	spi(0x700 + Ly);
	RS_0;
	spi(Color);
	#endif
	#endif

}

//######################################################
//#               Line zeichnen                        #
//######################################################

void S65_line(unsigned char Lx1, unsigned char Ly1, unsigned char Lx2, unsigned char Ly2, double Color)
/*{
	char M, Yg, R, B;
	double M1, M2, Cv;
	int Bn;
	
	
	M1 = Ly2 - Ly1;
	M2 = Lx2 - Lx1;
	Cv = M1 / M2;
	Bn = (int)Cv;
	M = (char)(Bn&0xFF);
	B = M * Lx1;
	B = -1 * B;
	//If Ly1 = Ly2 Then B = 0
	//If Lx1 = Lx2 Then B = 0
	B = B + Ly1;
	
	for(R=Lx1;R<=Lx2;R++){
	Yg = M * R;
	Yg = Yg + B;
	S65_pset(R, Yg, Color);
	}

}*/

{
   unsigned char  xtemp, ytemp;
   int addx, addy, dx, dy;
  signed long P;
   int i;
   dx = fabs(Lx2 - Lx1);
   dy = fabs(Ly2 - Ly1);
   xtemp = Lx1;
   ytemp = Ly1;

   if(Lx1 > Lx2)
      addx = -1;
   else
      addx = 1;
   if(Ly1 > Ly2)
      addy = -1;
   else
      addy = 1;

   if(dx >= dy)
   {
      P = 2*dy - dx;

      for(i=0; i<=dx; ++i)
      {
         S65_pset(xtemp,ytemp,Color);
         if(P < 0)
         {
            P += 2*dy;
            xtemp += addx;
         }
         else
         {
            P += 2*dy - 2*dx;
            xtemp += addx;
            ytemp += addy;
         }
      }
   }
   else
   {
      P = 2*dx - dy;

      for(i=0; i<=dy; ++i)
      {
         S65_pset(xtemp,ytemp,Color);
         if(P < 0)
         {
            P += 2*dx;
            ytemp += addy;
         }
         else
         {
            P += 2*dx - 2*dy;
            xtemp += addx;
            ytemp += addy;
         }
      }
   }
}


//==================================================//
// Title:       LCDHLine
// Description: Draws a horizontal line
//              from xs (x start), ys (ystart) for
//              length 'length' using color 'color'
//              xs: 0-131    ys: 0-131
//              No bounds checking
//==================================================//
void LCDVLine(unsigned char xs, unsigned char ys, unsigned char length, double color)
{
 long i;

	#ifdef LANDSCAPE
	RS_1;
	spi(0x600 + ys);
	spi(0x700 + xs);
	RS_0;
	#else
	#ifdef PORTRAIT
	RS_1;
	spi(0x600 + xs);
	spi(0x700 + ys);
	RS_0;
	#endif
	#endif


 for (i=0;i<length;i++)                             // Number of horizontal pixels
     spi(color);
}

//==================================================//
// Title:       LCDVLine
// Description: Draws a vertical line
//              from xs (x start), ys (ystart) for
//              length 'length' using color 'color'
//              xs: 0-131    ys: 0-131
//              No bounds checking
//==================================================//
void LCDHLine(unsigned char xs, unsigned char ys, unsigned char length, double color)
{
 long i;


 for (i=0;i<length;i++)                             // Number of vertical pixels
     {
		#ifdef LANDSCAPE
		RS_1;
		spi(0x600 + ys);
		spi(0x700 + xs);
		RS_0;
		#else
		#ifdef PORTRAIT
		RS_1;
		spi(0x600 + xs);
		spi(0x700 + ys);
		RS_0;
		#endif
		#endif
	
	     spi(color);
     }
}

void spi(unsigned int dat) 
{
g_spi=dat;
spix();
/*
	unsigned char i;
	                        
	CS_0;                            

	for (i = 0; i < 16; i ++)          // followed by the 16 bits of data
	{
		SCLK_0;                     
		if((dat << i) & 0x8000)
			SDATA_1;                  
		else
			SDATA_0;                   
		
		SCLK_1;                      
	}

	CS_1;  
*/                           
}


//  while (txt[i]!=0)
 // {
    // two possible text outputs, same lower left (0) upper left (90) corner pixel
//    put_char(10+i*CHAR_W,60,txt[i],1);  // 90 deg. rotated
//    lcd_putchar(45+i*CHAR_W,60,txt[i],1);   // 0 deg. rotated
//    i++;
 // }

//==================================================//
// Title:       LCDCharacter
// Description: Draws character '_char' to the display
//              from point x,y of font size 'Size' in
//              the color 'color'
//              x: 0-131    y: 0-131
//              No bounds checking
//==================================================//
void Character (unsigned char Size, unsigned char xc, unsigned char yc, unsigned char _char, double color)
{
 long i,j;
 long Data;
 long Mask;

// The fonts start at 0x20 so offset into the font
_char-=0x20;

// 5x7 Font-----------------------------------------//
if (Size==_5x7Font) 
{
 for (i=0;i<5;i++)                                  // 5 bytes per char
     {
      Data = Font5x7[_char][i];                     // Get a line of pixels
      Mask=1;
        
      for (j=0;j<8;j++)
          {
           if (Data&Mask)
              S65_pset (xc+i,yc+j,color);
           Mask<<=1;
          }
     }
}
else
// 8x10 Font----------------------------------------//
 if(Size==_8x10Font) 
{
 for (i=0;i<8;i++)                                  // 10 bytes per char
     {
      Data = Font8x10[_char][i];                    // Get a line of pixels
      Mask=1;
        
      for (j=0;j<10;j++)
          {
           if (Data&Mask)
              S65_pset (xc+i,yc+j,color);
           Mask<<=1;
          }
     }
 // Last 2 bytes are done differently to save memory
 // The last two bytes have been placed in separate code
 // blocks so that you can more easilly see what is happening
 Data = Font8x10[_char][8];     
 Mask=1;
for (i=0;i<4;i++)
    {
     for (j=0;j<2;j++)
        {
         if (Data&Mask)
             S65_pset (xc+i,yc+j+8,color);
         Mask<<=1;
        }
    }

 Data = Font8x10[_char][9];     
 Mask=1;
for (i=0;i<4;i++)
    {
     for (j=0;j<2;j++)
        {
         if (Data&Mask)
             S65_pset (xc+4+i,yc+j+8,color);
         Mask<<=1;
        }
    }
 }
}

//==================================================//
// Title:       LCDString
// Description: Draws a string of characters to the
//              display from the current column and
//              row settings, in the color 'color'
// Note:
// I have used Z as a terminating char. This was done
// just to test the code quickly. It's not a desirable
// soultion for a real application.
//
//==================================================//
void LCDString (unsigned char xc, unsigned char yc, unsigned char Size, long StrNo, double color)
{
 unsigned char _char=0;
 unsigned char count;

 count=0;

while(_char!='Z')  // String termination character
    {
     _char=Strings[StrNo][count++];
     if (_char!='Z')
        {
         Character (Size, xc,yc,_char,color);
         CursorX+=8;
        }
    }
}